@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2016 Canonical Ltd
+ * Copyright (C) 2014-2020 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -45,21 +45,14 @@ type MockBootloader struct {
 	InstallBootConfigResult bool
 	InstallBootConfigErr    error
 
-	RecoverySystemDir      string
-	RecoverySystemBootVars map[string]string
-
-	runKernelImageEnableKernelCalls     []snap.PlaceInfo
-	runKernelImageEnableTryKernelCalls  []snap.PlaceInfo
-	runKernelImageDisableTryKernelCalls []snap.PlaceInfo
-	runKernelImageEnabledKernel         snap.PlaceInfo
-	runKernelImageEnabledTryKernel      snap.PlaceInfo
-
-	runKernelImageMockedErrs     map[string]error
-	runKernelImageMockedNumCalls map[string]int
+	panicMethods map[string]bool
 }
 
-// ensure MockBootloader implements the Bootloader interface
+// ensure MockBootloader(s) implement the Bootloader interfaceces
 var _ bootloader.Bootloader = (*MockBootloader)(nil)
+var _ bootloader.RecoveryAwareBootloader = (*MockRecoveryAwareBootloader)(nil)
+var _ bootloader.ExtractedRunKernelImageBootloader = (*MockExtractedRunKernelImageBootloader)(nil)
+var _ bootloader.ExtractedRecoveryKernelImageBootloader = (*MockExtractedRecoveryKernelImageBootloader)(nil)
 
 func Mock(name, bootdir string) *MockBootloader {
 	return &MockBootloader{
@@ -68,12 +61,18 @@ func Mock(name, bootdir string) *MockBootloader {
 
 		BootVars: make(map[string]string),
 
-		runKernelImageMockedErrs:     make(map[string]error),
-		runKernelImageMockedNumCalls: make(map[string]int),
+		panicMethods: make(map[string]bool),
+	}
+}
+
+func (b *MockBootloader) maybePanic(which string) {
+	if b.panicMethods[which] {
+		panic(fmt.Sprintf("mocked reboot panic in %s", which))
 	}
 }
 
 func (b *MockBootloader) SetBootVars(values map[string]string) error {
+	b.maybePanic("SetBootVars")
 	b.SetBootVarsCalls++
 	for k, v := range values {
 		b.BootVars[k] = v
@@ -82,6 +81,8 @@ func (b *MockBootloader) SetBootVars(values map[string]string) error {
 }
 
 func (b *MockBootloader) GetBootVars(keys ...string) (map[string]string, error) {
+	b.maybePanic("GetBootVars")
+
 	out := map[string]string{}
 	for _, k := range keys {
 		out[k] = b.BootVars[k]
@@ -108,45 +109,6 @@ func (b *MockBootloader) RemoveKernelAssets(s snap.PlaceInfo) error {
 	return nil
 }
 
-// SetBootKernel sets the current boot kernel string. Should be
-// something like "pc-kernel_1234.snap".
-func (b *MockBootloader) SetBootKernel(kernel string) {
-	b.SetBootVars(map[string]string{"snap_kernel": kernel})
-}
-
-// SetBootBase sets the current boot base string. Should be something
-// like "core_1234.snap".
-func (b *MockBootloader) SetBootBase(base string) {
-	b.SetBootVars(map[string]string{"snap_core": base})
-}
-
-func (b *MockBootloader) SetTryingDuringReboot() error {
-	if b.BootVars["snap_mode"] != "try" {
-		return fmt.Errorf("bootloader must be in 'try' mode")
-	}
-	b.BootVars["snap_mode"] = "trying"
-	return nil
-}
-
-// SetRollbackAcrossReboot will simulate a rollback across reboots. This
-// means that the bootloader had "snap_try_{core,kernel}" set but this
-// boot failed. In this case the bootloader will clear
-// "snap_try_{core,kernel}" and "snap_mode" which means the "old" kernel,core
-// in "snap_{core,kernel}" will be used.
-func (b *MockBootloader) SetRollbackAcrossReboot() error {
-	if b.BootVars["snap_mode"] != "try" {
-		return fmt.Errorf("rollback can only be simulated in 'try' mode")
-	}
-	if b.BootVars["snap_core"] == "" && b.BootVars["snap_kernel"] == "" {
-		return fmt.Errorf("rollback can only be simulated if either snap_core or snap_kernel is set")
-	}
-	// clean try bootvars and snap_mode
-	b.BootVars["snap_mode"] = ""
-	b.BootVars["snap_try_core"] = ""
-	b.BootVars["snap_try_kernel"] = ""
-	return nil
-}
-
 // InstallBootConfig installs the boot config in the gadget directory to the
 // mock bootloader's root directory.
 func (b *MockBootloader) InstallBootConfig(gadgetDir string, opts *bootloader.Options) (bool, error) {
@@ -154,9 +116,60 @@ func (b *MockBootloader) InstallBootConfig(gadgetDir string, opts *bootloader.Op
 	return b.InstallBootConfigResult, b.InstallBootConfigErr
 }
 
+// MockRecoveryAwareBootloader mocks a bootloader implementing the
+// RecoveryAware interface.
+type MockRecoveryAwareBootloader struct {
+	*MockBootloader
+
+	RecoverySystemDir      string
+	RecoverySystemBootVars map[string]string
+}
+
+type ExtractedRecoveryKernelCall struct {
+	RecoverySystemDir string
+	S                 snap.PlaceInfo
+}
+
+// MockExtractedRecoveryKernelImageBootloader mocks a bootloader implementing
+// the ExtractedRecoveryKernelImage interface.
+type MockExtractedRecoveryKernelImageBootloader struct {
+	*MockBootloader
+
+	ExtractRecoveryKernelAssetsCalls []ExtractedRecoveryKernelCall
+}
+
+// ExtractedRecoveryKernelImage derives a MockRecoveryAwareBootloader from a base
+// MockBootloader.
+func (b *MockBootloader) ExtractedRecoveryKernelImage() *MockExtractedRecoveryKernelImageBootloader {
+	return &MockExtractedRecoveryKernelImageBootloader{MockBootloader: b}
+}
+
+// ExtractRecoveryKernelAssets extracts the kernel assets for the provided
+// kernel snap into the specified recovery system dir; part of
+// RecoveryAwareBootloader.
+func (b *MockExtractedRecoveryKernelImageBootloader) ExtractRecoveryKernelAssets(recoverySystemDir string, s snap.PlaceInfo, snapf snap.Container) error {
+	if recoverySystemDir == "" {
+		panic("MockBootloader.ExtractRecoveryKernelAssets called without recoverySystemDir")
+	}
+
+	b.ExtractRecoveryKernelAssetsCalls = append(
+		b.ExtractRecoveryKernelAssetsCalls,
+		ExtractedRecoveryKernelCall{
+			S:                 s,
+			RecoverySystemDir: recoverySystemDir},
+	)
+	return nil
+}
+
+// RecoveryAware derives a MockRecoveryAwareBootloader from a base
+// MockBootloader.
+func (b *MockBootloader) RecoveryAware() *MockRecoveryAwareBootloader {
+	return &MockRecoveryAwareBootloader{MockBootloader: b}
+}
+
 // SetRecoverySystemEnv sets the recovery system environment bootloader
 // variables; part of RecoveryAwareBootloader.
-func (b *MockBootloader) SetRecoverySystemEnv(recoverySystemDir string, blVars map[string]string) error {
+func (b *MockRecoveryAwareBootloader) SetRecoverySystemEnv(recoverySystemDir string, blVars map[string]string) error {
 	if recoverySystemDir == "" {
 		panic("MockBootloader.SetRecoverySystemEnv called without recoverySystemDir")
 	}
@@ -165,10 +178,36 @@ func (b *MockBootloader) SetRecoverySystemEnv(recoverySystemDir string, blVars m
 	return nil
 }
 
+// MockExtractedRunKernelImageBootloader mocks a bootloader
+// implementing the ExtractedRunKernelImageBootloader interface.
+type MockExtractedRunKernelImageBootloader struct {
+	*MockBootloader
+
+	runKernelImageEnableKernelCalls     []snap.PlaceInfo
+	runKernelImageEnableTryKernelCalls  []snap.PlaceInfo
+	runKernelImageDisableTryKernelCalls []snap.PlaceInfo
+	runKernelImageEnabledKernel         snap.PlaceInfo
+	runKernelImageEnabledTryKernel      snap.PlaceInfo
+
+	runKernelImageMockedErrs     map[string]error
+	runKernelImageMockedNumCalls map[string]int
+}
+
+// WithExtractedRunKernelImage derives a MockExtractedRunKernelImageBootloader
+// from a base MockBootloader.
+func (b *MockBootloader) WithExtractedRunKernelImage() *MockExtractedRunKernelImageBootloader {
+	return &MockExtractedRunKernelImageBootloader{
+		MockBootloader: b,
+
+		runKernelImageMockedErrs:     make(map[string]error),
+		runKernelImageMockedNumCalls: make(map[string]int),
+	}
+}
+
 // SetRunKernelImageEnabledKernel sets the current kernel "symlink" as returned
 // by Kernel(); returns' a restore function to set it back to what it was
 // before.
-func (b *MockBootloader) SetRunKernelImageEnabledKernel(kernel snap.PlaceInfo) (restore func()) {
+func (b *MockExtractedRunKernelImageBootloader) SetRunKernelImageEnabledKernel(kernel snap.PlaceInfo) (restore func()) {
 	old := b.runKernelImageEnabledKernel
 	b.runKernelImageEnabledKernel = kernel
 	return func() {
@@ -180,7 +219,7 @@ func (b *MockBootloader) SetRunKernelImageEnabledKernel(kernel snap.PlaceInfo) (
 // returned by TryKernel(). If set to nil, TryKernel()'s second return value
 // will be false; returns' a restore function to set it back to what it was
 // before.
-func (b *MockBootloader) SetRunKernelImageEnabledTryKernel(kernel snap.PlaceInfo) (restore func()) {
+func (b *MockExtractedRunKernelImageBootloader) SetRunKernelImageEnabledTryKernel(kernel snap.PlaceInfo) (restore func()) {
 	old := b.runKernelImageEnabledTryKernel
 	b.runKernelImageEnabledTryKernel = kernel
 	return func() {
@@ -191,7 +230,7 @@ func (b *MockBootloader) SetRunKernelImageEnabledTryKernel(kernel snap.PlaceInfo
 // SetRunKernelImageFunctionError allows setting an error to be returned for the
 // specified function; it returns a restore function to set it back to what it
 // was before.
-func (b *MockBootloader) SetRunKernelImageFunctionError(f string, err error) (restore func()) {
+func (b *MockExtractedRunKernelImageBootloader) SetRunKernelImageFunctionError(f string, err error) (restore func()) {
 	// check the function
 	switch f {
 	case "EnableKernel", "EnableTryKernel", "Kernel", "TryKernel", "DisableTryKernel":
@@ -205,10 +244,31 @@ func (b *MockBootloader) SetRunKernelImageFunctionError(f string, err error) (re
 	}
 }
 
+// SetRunKernelImagePanic allows setting any method in the
+// ExtractedRunKernelImageBootloader interface on
+// MockExtractedRunKernelImageBootloader to panic instead of
+// returning. This allows one to test what would happen if the system
+// was rebooted during execution of a particular
+// function. Specifically, the panic will be done immediately entering
+// the function so setting SetBootVars to panic will emulate a reboot
+// before any boot vars are set persistently
+func (b *MockExtractedRunKernelImageBootloader) SetRunKernelImagePanic(f string) (restore func()) {
+	switch f {
+	case "EnableKernel", "EnableTryKernel", "Kernel", "TryKernel", "DisableTryKernel", "SetBootVars", "GetBootVars":
+		old := b.panicMethods[f]
+		b.panicMethods[f] = true
+		return func() {
+			b.panicMethods[f] = old
+		}
+	default:
+		panic(fmt.Sprintf("unknown ExtractedRunKernelImageBootloader method %q to mock reboot via panic for", f))
+	}
+}
+
 // GetRunKernelImageFunctionSnapCalls returns which snaps were specified during
 // execution, in order of calls, as well as the number of calls for methods that
 // don't take a snap to set.
-func (b *MockBootloader) GetRunKernelImageFunctionSnapCalls(f string) ([]snap.PlaceInfo, int) {
+func (b *MockExtractedRunKernelImageBootloader) GetRunKernelImageFunctionSnapCalls(f string) ([]snap.PlaceInfo, int) {
 	switch f {
 	case "EnableKernel":
 		l := b.runKernelImageEnableKernelCalls
@@ -224,21 +284,26 @@ func (b *MockBootloader) GetRunKernelImageFunctionSnapCalls(f string) ([]snap.Pl
 }
 
 // EnableKernel enables the kernel; part of ExtractedRunKernelImageBootloader.
-func (b *MockBootloader) EnableKernel(s snap.PlaceInfo) error {
+func (b *MockExtractedRunKernelImageBootloader) EnableKernel(s snap.PlaceInfo) error {
+	b.maybePanic("EnableKernel")
 	b.runKernelImageEnableKernelCalls = append(b.runKernelImageEnableKernelCalls, s)
+	b.runKernelImageEnabledKernel = s
 	return b.runKernelImageMockedErrs["EnableKernel"]
 }
 
 // EnableTryKernel enables a try-kernel; part of
 // ExtractedRunKernelImageBootloader.
-func (b *MockBootloader) EnableTryKernel(s snap.PlaceInfo) error {
+func (b *MockExtractedRunKernelImageBootloader) EnableTryKernel(s snap.PlaceInfo) error {
+	b.maybePanic("EnableTryKernel")
 	b.runKernelImageEnableTryKernelCalls = append(b.runKernelImageEnableTryKernelCalls, s)
+	b.runKernelImageEnabledTryKernel = s
 	return b.runKernelImageMockedErrs["EnableTryKernel"]
 }
 
 // Kernel returns the current kernel set in the bootloader; part of
 // ExtractedRunKernelImageBootloader.
-func (b *MockBootloader) Kernel() (snap.PlaceInfo, error) {
+func (b *MockExtractedRunKernelImageBootloader) Kernel() (snap.PlaceInfo, error) {
+	b.maybePanic("Kernel")
 	b.runKernelImageMockedNumCalls["Kernel"]++
 	err := b.runKernelImageMockedErrs["Kernel"]
 	if err != nil {
@@ -249,21 +314,24 @@ func (b *MockBootloader) Kernel() (snap.PlaceInfo, error) {
 
 // TryKernel returns the current kernel set in the bootloader; part of
 // ExtractedRunKernelImageBootloader.
-func (b *MockBootloader) TryKernel() (snap.PlaceInfo, bool, error) {
+func (b *MockExtractedRunKernelImageBootloader) TryKernel() (snap.PlaceInfo, error) {
+	b.maybePanic("TryKernel")
 	b.runKernelImageMockedNumCalls["TryKernel"]++
 	err := b.runKernelImageMockedErrs["TryKernel"]
 	if err != nil {
-		return nil, false, err
+		return nil, err
 	}
 	if b.runKernelImageEnabledTryKernel == nil {
-		return nil, false, nil
+		return nil, bootloader.ErrNoTryKernelRef
 	}
-	return b.runKernelImageEnabledTryKernel, true, nil
+	return b.runKernelImageEnabledTryKernel, nil
 }
 
 // DisableTryKernel removes the current try-kernel "symlink" set in the
 // bootloader; part of ExtractedRunKernelImageBootloader.
-func (b *MockBootloader) DisableTryKernel() error {
+func (b *MockExtractedRunKernelImageBootloader) DisableTryKernel() error {
+	b.maybePanic("DisableTryKernel")
 	b.runKernelImageMockedNumCalls["DisableTryKernel"]++
+	b.runKernelImageEnabledTryKernel = nil
 	return b.runKernelImageMockedErrs["DisableTryKernel"]
 }
