@@ -31,11 +31,41 @@ import (
 
 type uboot struct {
 	rootdir string
+	basedir string
+
+	ubootEnvFileName string
+}
+
+func (u *uboot) setDefaults() {
+	u.basedir = "/boot/uboot/"
+	u.ubootEnvFileName = "uboot.env"
+}
+
+func (u *uboot) processBlOpts(blOpts *Options) {
+	if blOpts != nil {
+		switch {
+		case blOpts.NoSlashBoot, blOpts.Recovery:
+			// Recovery or NoSlashBoot imply we use the "boot.sel" simple text
+			// format file in /uboot/ubuntu as it exists on the partition
+			// directly
+			u.basedir = "/uboot/ubuntu/"
+			fallthrough
+		case blOpts.ExtractedRunKernelImage:
+			// if just ExtractedRunKernelImage is defined, we expect to find
+			// /boot/uboot/boot.sel
+			u.ubootEnvFileName = "boot.sel"
+		}
+	}
 }
 
 // newUboot create a new Uboot bootloader object
-func newUboot(rootdir string) RecoveryAwareBootloader {
-	u := &uboot{rootdir: rootdir}
+func newUboot(rootdir string, blOpts *Options) ExtractedRecoveryKernelImageBootloader {
+	u := &uboot{
+		rootdir: rootdir,
+	}
+	u.setDefaults()
+	u.processBlOpts(blOpts)
+
 	if !osutil.FileExists(u.envFile()) {
 		return nil
 	}
@@ -55,43 +85,66 @@ func (u *uboot) dir() string {
 	if u.rootdir == "" {
 		panic("internal error: unset rootdir")
 	}
-	return filepath.Join(u.rootdir, "/boot/uboot")
+	return filepath.Join(u.rootdir, u.basedir)
 }
 
-func (u *uboot) InstallBootConfig(gadgetDir string, opts *Options) (bool, error) {
+func (u *uboot) InstallBootConfig(gadgetDir string, blOpts *Options) (bool, error) {
 	gadgetFile := filepath.Join(gadgetDir, u.Name()+".conf")
+	// if the gadget file is empty, then we don't install anything
+	// this is because there are some gadgets, namely the 20 pi gadget right
+	// now, that don't use a uboot.env to boot and instead use a boot.scr, and
+	// installing a uboot.env file of any form in the root directory will break
+	// the boot.scr, so for these setups we just don't install anything
+	// TODO:UC20: how can we do this better? maybe parse the file to get the
+	//            actual format?
+	st, err := os.Stat(gadgetFile)
+	if err != nil {
+		return false, err
+	}
+	if st.Size() == 0 {
+		// we have an empty uboot.conf, and hence a uboot bootloader in the
+		// gadget, but nothing to copy in this case and instead just install our
+		// own boot.sel file
+		u.processBlOpts(blOpts)
+
+		err := os.MkdirAll(filepath.Dir(u.envFile()), 0755)
+		if err != nil {
+			return false, err
+		}
+
+		// TODO:UC20: what's a reasonable size for this file?
+		env, err := ubootenv.Create(u.envFile(), 4096)
+		if err != nil {
+			return false, err
+		}
+
+		if err := env.Save(); err != nil {
+			return false, nil
+		}
+
+		return true, nil
+	}
+
+	// InstallBootConfig gets called on a uboot that does not come from newUboot
+	// so we need to apply the defaults here
+	u.setDefaults()
+
+	if blOpts != nil && blOpts.Recovery {
+		// not supported yet, this is traditional uboot.env from gadget
+		// TODO:UC20: support this use-case
+		return false, fmt.Errorf("non-empty uboot.env not supported on UC20 yet")
+	}
+
 	systemFile := u.ConfigFile()
 	return genericInstallBootConfig(gadgetFile, systemFile)
 }
 
-func (u *uboot) SetRecoverySystemEnv(recoverySystemDir string, values map[string]string) error {
-	if recoverySystemDir == "" {
-		return fmt.Errorf("internal error: recoverySystemDir unset")
-	}
-	recoverySystemUbootEnv := filepath.Join(u.rootdir, recoverySystemDir, "uboot.env")
-	if err := os.MkdirAll(filepath.Dir(recoverySystemUbootEnv), 0755); err != nil {
-		return err
-	}
-	// XXX: Hardcoded size. We only store the
-	// snapd_recovery_kernel line here so this size is more than
-	// enough
-	uenvSize := 8192
-	uenv, err := ubootenv.Create(recoverySystemUbootEnv, uenvSize)
-	if err != nil {
-		return err
-	}
-	for k, v := range values {
-		uenv.Set(k, v)
-	}
-	return uenv.Save()
-}
-
 func (u *uboot) ConfigFile() string {
 	return u.envFile()
 }
 
 func (u *uboot) envFile() string {
-	return filepath.Join(u.dir(), "uboot.env")
+	return filepath.Join(u.dir(), u.ubootEnvFileName)
 }
 
 func (u *uboot) SetBootVars(values map[string]string) error {
@@ -135,7 +188,17 @@ func (u *uboot) GetBootVars(names ...string) (map[string]string, error) {
 func (u *uboot) ExtractKernelAssets(s snap.PlaceInfo, snapf snap.Container) error {
 	dstDir := filepath.Join(u.dir(), s.Filename())
 	assets := []string{"kernel.img", "initrd.img", "dtbs/*"}
-	return extractKernelAssetsToBootDir(dstDir, s, snapf, assets)
+	return extractKernelAssetsToBootDir(dstDir, snapf, assets)
+}
+
+func (u *uboot) ExtractRecoveryKernelAssets(recoverySystemDir string, s snap.PlaceInfo, snapf snap.Container) error {
+	if recoverySystemDir == "" {
+		return fmt.Errorf("internal error: recoverySystemDir unset")
+	}
+
+	recoverySystemUbootKernelAssetsDir := filepath.Join(u.rootdir, recoverySystemDir, "kernel")
+	assets := []string{"kernel.img", "initrd.img", "dtbs/*"}
+	return extractKernelAssetsToBootDir(recoverySystemUbootKernelAssetsDir, snapf, assets)
 }
 
 func (u *uboot) RemoveKernelAssets(s snap.PlaceInfo) error {
