@@ -26,6 +26,7 @@ import (
 	"path/filepath"
 
 	"github.com/snapcore/snapd/dirs"
+	apparmor_sandbox "github.com/snapcore/snapd/sandbox/apparmor"
 )
 
 func resetPreseededChroot(preseedChroot string) error {
@@ -61,7 +62,7 @@ func resetPreseededChroot(preseedChroot string) error {
 	globs = []string{
 		filepath.Join(dirs.SnapDataDir, "*"),
 		filepath.Join(dirs.SnapCacheDir, "*"),
-		filepath.Join(dirs.AppArmorCacheDir, "*"),
+		filepath.Join(apparmor_sandbox.CacheDir, "*"),
 		filepath.Join(dirs.SnapDesktopFilesDir, "*"),
 	}
 
