@@ -39,6 +39,7 @@ import (
 	"github.com/snapcore/snapd/i18n"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snap/snapfile"
 	"github.com/snapcore/snapd/snap/squashfs"
 	"github.com/snapcore/snapd/strutil"
 )
@@ -112,7 +113,7 @@ func (iw *infoWriter) maybePrintHealth() {
 }
 
 func clientSnapFromPath(path string) (*client.Snap, error) {
-	snapf, err := snap.Open(path)
+	snapf, err := snapfile.Open(path)
 	if err != nil {
 		return nil, err
 	}
@@ -207,11 +208,10 @@ func wrapGeneric(out io.Writer, text []rune, indent, indent2 string, termWidth i
 	width := termWidth - indentWidth
 
 	// establish the indent of the whole block
-	idx := 0
 	var err error
 	for len(text) > width && err == nil {
 		// find a good place to chop the text
-		idx = runesLastIndexSpace(text[:width+1])
+		idx := runesLastIndexSpace(text[:width+1])
 		if idx < 0 {
 			// there's no whitespace; just chop at line width
 			idx = width
@@ -401,6 +401,23 @@ func (iw *infoWriter) printSummary() {
 	wrapFlow(iw, quotedIfNeeded(iw.theSnap.Summary), "summary:\t", iw.termWidth)
 }
 
+func (iw *infoWriter) maybePrintStoreURL() {
+	storeURL := ""
+	// XXX: store-url for local snaps comes from aux data, but that gets
+	// updated only when the snap is refreshed, be smart and poke remote
+	// snap info if available
+	switch {
+	case iw.theSnap.StoreURL != "":
+		storeURL = iw.theSnap.StoreURL
+	case iw.remoteSnap != nil && iw.remoteSnap.StoreURL != "":
+		storeURL = iw.remoteSnap.StoreURL
+	}
+	if storeURL == "" {
+		return
+	}
+	fmt.Fprintf(iw, "store-url:\t%s\n", storeURL)
+}
+
 func (iw *infoWriter) maybePrintPublisher() {
 	if iw.diskSnap != nil {
 		// snaps read from disk won't have a publisher
@@ -639,9 +656,6 @@ func (chInfos *channelInfos) addFromRemote(remote *client.Snap) {
 		for _, risk := range channelRisks {
 			chName := fmt.Sprintf("%s/%s", tr, risk)
 			ch, ok := remote.Channels[chName]
-			if tr == "latest" {
-				chName = risk
-			}
 			if ok {
 				chInfos.addOpenChannel(chName, ch.Version, ch.Revision, ch.ReleasedAt, ch.Size, NotesFromChannelSnapInfo(ch))
 				trackHasOpenChannel = true
@@ -717,6 +731,7 @@ func (x *infoCmd) Execute([]string) error {
 		iw.printSummary()
 		iw.maybePrintHealth()
 		iw.maybePrintPublisher()
+		iw.maybePrintStoreURL()
 		iw.maybePrintStandaloneVersion()
 		iw.maybePrintBuildDate()
 		iw.maybePrintContact()
