@@ -115,11 +115,14 @@ and environment.
 // isStopping returns true if the system is shutting down.
 func isStopping() (bool, error) {
 	// Make sure, just in case, that systemd doesn't localize the output string.
-	env := os.Environ()
-	env = append(env, "LC_MESSAGES=C")
+	env, err := osutil.OSEnvironment()
+	if err != nil {
+		return false, err
+	}
+	env["LC_MESSAGES"] = "C"
 	// Check if systemd is stopping (shutting down or rebooting).
 	cmd := exec.Command("systemctl", "is-system-running")
-	cmd.Env = env
+	cmd.Env = env.ForExec()
 	stdout, err := cmd.Output()
 	// systemctl is-system-running returns non-zero for outcomes other than "running"
 	// As such, ignore any ExitError and just process the stdout buffer.
@@ -743,9 +746,9 @@ func activateXdgDocumentPortal(info *snap.Info, snapApp, hook string) error {
 	return nil
 }
 
-func (x *cmdRun) runCmdUnderGdb(origCmd, env []string) error {
-	env = append(env, "SNAP_CONFINE_RUN_UNDER_GDB=1")
+type envForExecFunc func(extra map[string]string) []string
 
+func (x *cmdRun) runCmdUnderGdb(origCmd []string, envForExec envForExecFunc) error {
 	cmd := []string{"sudo", "-E", "gdb", "-ex=run", "-ex=catch exec", "-ex=continue", "--args"}
 	cmd = append(cmd, origCmd...)
 
@@ -753,11 +756,11 @@ func (x *cmdRun) runCmdUnderGdb(origCmd, env []string) error {
 	gcmd.Stdin = os.Stdin
 	gcmd.Stdout = os.Stdout
 	gcmd.Stderr = os.Stderr
-	gcmd.Env = env
+	gcmd.Env = envForExec(map[string]string{"SNAP_CONFINE_RUN_UNDER_GDB": "1"})
 	return gcmd.Run()
 }
 
-func (x *cmdRun) runCmdWithTraceExec(origCmd, env []string) error {
+func (x *cmdRun) runCmdWithTraceExec(origCmd []string, envForExec envForExecFunc) error {
 	// setup private tmp dir with strace fifo
 	straceTmp, err := ioutil.TempDir("", "exec-trace")
 	if err != nil {
@@ -792,7 +795,7 @@ func (x *cmdRun) runCmdWithTraceExec(origCmd, env []string) error {
 		return err
 	}
 	// run
-	cmd.Env = env
+	cmd.Env = envForExec(nil)
 	cmd.Stdin = Stdin
 	cmd.Stdout = Stdout
 	cmd.Stderr = Stderr
@@ -812,7 +815,7 @@ func (x *cmdRun) runCmdWithTraceExec(origCmd, env []string) error {
 	return err
 }
 
-func (x *cmdRun) runCmdUnderStrace(origCmd, env []string) error {
+func (x *cmdRun) runCmdUnderStrace(origCmd []string, envForExec envForExecFunc) error {
 	extraStraceOpts, raw, err := x.straceOpts()
 	if err != nil {
 		return err
@@ -823,7 +826,7 @@ func (x *cmdRun) runCmdUnderStrace(origCmd, env []string) error {
 	}
 
 	// run with filter
-	cmd.Env = env
+	cmd.Env = envForExec(nil)
 	cmd.Stdin = Stdin
 	cmd.Stdout = Stdout
 	stderr, err := cmd.StderrPipe()
@@ -969,19 +972,44 @@ func (x *cmdRun) runSnapConfine(info *snap.Info, securityTag, snapApp, hook stri
 	cmd = append(cmd, snapApp)
 	cmd = append(cmd, args...)
 
-	extraEnv := make(map[string]string)
+	env, err := osutil.OSEnvironment()
+	if err != nil {
+		return err
+	}
+	snapenv.ExtendEnvForRun(env, info)
+
 	if len(xauthPath) > 0 {
-		extraEnv["XAUTHORITY"] = xauthPath
+		// Environment is not nil here because it comes from
+		// osutil.OSEnvironment and that guarantees this
+		// property.
+		env["XAUTHORITY"] = xauthPath
+	}
+
+	// on each run variant path this will be used once to get
+	// the environment plus additions in the right form
+	envForExec := func(extra map[string]string) []string {
+		for varName, value := range extra {
+			env[varName] = value
+		}
+		if !info.NeedsClassic() {
+			return env.ForExec()
+		}
+		// For a classic snap, environment variables that are
+		// usually stripped out by ld.so when starting a
+		// setuid process are presevered by being renamed by
+		// prepending PreservedUnsafePrefix -- which snap-exec
+		// will remove, restoring the variables to their
+		// original names.
+		return env.ForExecEscapeUnsafe(snapenv.PreservedUnsafePrefix)
 	}
-	env := snapenv.ExecEnv(info, extraEnv)
 
 	if x.TraceExec {
-		return x.runCmdWithTraceExec(cmd, env)
+		return x.runCmdWithTraceExec(cmd, envForExec)
 	} else if x.Gdb {
-		return x.runCmdUnderGdb(cmd, env)
+		return x.runCmdUnderGdb(cmd, envForExec)
 	} else if x.useStrace() {
-		return x.runCmdUnderStrace(cmd, env)
+		return x.runCmdUnderStrace(cmd, envForExec)
 	} else {
-		return syscallExec(cmd[0], cmd, env)
+		return syscallExec(cmd[0], cmd, envForExec(nil))
 	}
 }
