@@ -21,6 +21,7 @@ package daemon
 
 import (
 	"net/http"
+	"time"
 
 	"github.com/snapcore/snapd/overlord"
 )
@@ -51,3 +52,11 @@ func MockBuildID(mock string) (restore func()) {
 		buildID = old
 	}
 }
+
+func MockShutdownTimeout(tm time.Duration) (restore func()) {
+	old := shutdownTimeout
+	shutdownTimeout = tm
+	return func() {
+		shutdownTimeout = old
+	}
+}
