@@ -82,12 +82,6 @@ func (d *deviceSuite) setupMockSysfsForDevMapper(c *C) {
 	c.Assert(err, IsNil)
 }
 
-func (d *deviceSuite) setUpWritableFallback(c *C, mountInfo string) {
-	d.setupMockSysfs(c)
-
-	mockProcSelfFilesystem(c, d.dir, mountInfo)
-}
-
 func (d *deviceSuite) TestDeviceFindByStructureName(c *C) {
 	names := []struct {
 		escaped   string
@@ -146,7 +140,10 @@ func (d *deviceSuite) TestDeviceFindByFilesystemLabel(c *C) {
 	for _, tc := range names {
 		c.Logf("trying: %q", tc)
 		found, err := gadget.FindDeviceForStructure(&gadget.LaidOutStructure{
-			VolumeStructure: &gadget.VolumeStructure{Label: tc.structure},
+			VolumeStructure: &gadget.VolumeStructure{
+				Filesystem: "ext4",
+				Label:      tc.structure,
+			},
 		})
 		c.Check(err, IsNil)
 		c.Check(found, Equals, filepath.Join(d.dir, "/dev/fakedevice"))
@@ -171,6 +168,22 @@ func (d *deviceSuite) TestDeviceFindChecksPartlabelAndFilesystemLabelHappy(c *C)
 	c.Check(found, Equals, filepath.Join(d.dir, "/dev/fakedevice"))
 }
 
+func (d *deviceSuite) TestDeviceFindFilesystemLabelToNameFallback(c *C) {
+	fakedevice := filepath.Join(d.dir, "/dev/fakedevice")
+	// only the by-filesystem-label symlink
+	err := os.Symlink(fakedevice, filepath.Join(d.dir, "/dev/disk/by-label/foo"))
+	c.Assert(err, IsNil)
+
+	found, err := gadget.FindDeviceForStructure(&gadget.LaidOutStructure{
+		VolumeStructure: &gadget.VolumeStructure{
+			Name:       "foo",
+			Filesystem: "ext4",
+		},
+	})
+	c.Check(err, IsNil)
+	c.Check(found, Equals, filepath.Join(d.dir, "/dev/fakedevice"))
+}
+
 func (d *deviceSuite) TestDeviceFindChecksPartlabelAndFilesystemLabelMismatch(c *C) {
 	fakedevice := filepath.Join(d.dir, "/dev/fakedevice")
 	err := os.Symlink(fakedevice, filepath.Join(d.dir, "/dev/disk/by-label/foo"))
@@ -185,8 +198,9 @@ func (d *deviceSuite) TestDeviceFindChecksPartlabelAndFilesystemLabelMismatch(c
 
 	found, err := gadget.FindDeviceForStructure(&gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
-			Name:  "bar",
-			Label: "foo",
+			Name:       "bar",
+			Label:      "foo",
+			Filesystem: "ext4",
 		},
 	})
 	c.Check(err, ErrorMatches, `conflicting device match, ".*/by-label/foo" points to ".*/fakedevice", previous match ".*/by-partlabel/bar" points to ".*/fakedevice-other"`)
@@ -208,12 +222,25 @@ func (d *deviceSuite) TestDeviceFindNotFoundEmpty(c *C) {
 	// neither name nor filesystem label set
 	found, err := gadget.FindDeviceForStructure(&gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
-			Name:  "",
+			Name: "",
+			// structure has no filesystem, fs label check is
+			// ineffective
 			Label: "",
 		},
 	})
 	c.Check(err, ErrorMatches, `device not found`)
 	c.Check(found, Equals, "")
+
+	// try with proper filesystem now
+	found, err = gadget.FindDeviceForStructure(&gadget.LaidOutStructure{
+		VolumeStructure: &gadget.VolumeStructure{
+			Name:       "",
+			Label:      "",
+			Filesystem: "ext4",
+		},
+	})
+	c.Check(err, ErrorMatches, `device not found`)
+	c.Check(found, Equals, "")
 }
 
 func (d *deviceSuite) TestDeviceFindNotFoundSymlinkPointsNowhere(c *C) {
@@ -236,7 +263,8 @@ func (d *deviceSuite) TestDeviceFindNotFoundNotASymlink(c *C) {
 
 	found, err := gadget.FindDeviceForStructure(&gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
-			Label: "foo",
+			Filesystem: "ext4",
+			Label:      "foo",
 		},
 	})
 	c.Check(err, ErrorMatches, `candidate .*/dev/disk/by-label/foo is not a symlink`)
@@ -257,7 +285,8 @@ func (d *deviceSuite) TestDeviceFindBadEvalSymlinks(c *C) {
 
 	found, err := gadget.FindDeviceForStructure(&gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
-			Label: "foo",
+			Filesystem: "vfat",
+			Label:      "foo",
 		},
 	})
 	c.Check(err, ErrorMatches, `cannot read device link: failed`)
@@ -268,8 +297,8 @@ var writableMountInfo = `26 27 8:3 / /writable rw,relatime shared:7 - ext4 /dev/
 
 func (d *deviceSuite) TestDeviceFindFallbackNotFoundNoWritable(c *C) {
 	badMountInfo := `26 27 8:3 / /not-writable rw,relatime shared:7 - ext4 /dev/fakedevice0p1 rw,data=ordered`
-
-	mockProcSelfFilesystem(c, d.dir, badMountInfo)
+	restore := osutil.MockMountInfo(badMountInfo)
+	defer restore()
 
 	found, offs, err := gadget.FindDeviceForStructureWithFallback(&gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
@@ -283,7 +312,8 @@ func (d *deviceSuite) TestDeviceFindFallbackNotFoundNoWritable(c *C) {
 }
 
 func (d *deviceSuite) TestDeviceFindFallbackBadWritable(c *C) {
-	mockProcSelfFilesystem(c, d.dir, writableMountInfo)
+	restore := osutil.MockMountInfo(writableMountInfo)
+	defer restore()
 
 	ps := &gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
@@ -307,7 +337,9 @@ func (d *deviceSuite) TestDeviceFindFallbackBadWritable(c *C) {
 }
 
 func (d *deviceSuite) TestDeviceFindFallbackHappyWritable(c *C) {
-	d.setUpWritableFallback(c, writableMountInfo)
+	d.setupMockSysfs(c)
+	restore := osutil.MockMountInfo(writableMountInfo)
+	defer restore()
 
 	psJustBare := &gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
@@ -322,21 +354,34 @@ func (d *deviceSuite) TestDeviceFindFallbackHappyWritable(c *C) {
 		},
 		StartOffset: 123,
 	}
+	psMBR := &gadget.LaidOutStructure{
+		VolumeStructure: &gadget.VolumeStructure{
+			Type: "mbr",
+			Name: "mbr",
+		},
+		StartOffset: 0,
+	}
 	psNoName := &gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{},
 		StartOffset:     123,
 	}
 
-	for _, ps := range []*gadget.LaidOutStructure{psJustBare, psBareWithName, psNoName} {
+	for _, ps := range []*gadget.LaidOutStructure{psJustBare, psBareWithName, psNoName, psMBR} {
 		found, offs, err := gadget.FindDeviceForStructureWithFallback(ps)
 		c.Check(err, IsNil)
 		c.Check(found, Equals, filepath.Join(d.dir, "/dev/fakedevice0"))
-		c.Check(offs, Equals, gadget.Size(123))
+		if ps.Type != "mbr" {
+			c.Check(offs, Equals, gadget.Size(123))
+		} else {
+			c.Check(offs, Equals, gadget.Size(0))
+		}
 	}
 }
 
 func (d *deviceSuite) TestDeviceFindFallbackNotForNamedWritable(c *C) {
-	d.setUpWritableFallback(c, writableMountInfo)
+	d.setupMockSysfs(c)
+	restore := osutil.MockMountInfo(writableMountInfo)
+	defer restore()
 
 	// should not hit the fallback path
 	psNamed := &gadget.LaidOutStructure{
@@ -352,7 +397,9 @@ func (d *deviceSuite) TestDeviceFindFallbackNotForNamedWritable(c *C) {
 }
 
 func (d *deviceSuite) TestDeviceFindFallbackNotForFilesystem(c *C) {
-	d.setUpWritableFallback(c, writableMountInfo)
+	d.setupMockSysfs(c)
+	restore := osutil.MockMountInfo(writableMountInfo)
+	defer restore()
 
 	psFs := &gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
@@ -368,7 +415,9 @@ func (d *deviceSuite) TestDeviceFindFallbackNotForFilesystem(c *C) {
 }
 
 func (d *deviceSuite) TestDeviceFindFallbackBadMountInfo(c *C) {
-	d.setUpWritableFallback(c, "garbage")
+	d.setupMockSysfs(c)
+	restore := osutil.MockMountInfo("garbage")
+	defer restore()
 	psFs := &gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
 			Name: "foo",
@@ -488,14 +537,6 @@ func (d *deviceSuite) TestDeviceFindMountPointErrorsFromDevice(c *C) {
 	c.Check(p, Equals, "")
 }
 
-func mockProcSelfFilesystem(c *C, root, content string) {
-	psmi := filepath.Join(root, osutil.ProcSelfMountInfo)
-	err := os.MkdirAll(filepath.Dir(psmi), 0755)
-	c.Assert(err, IsNil)
-	err = ioutil.WriteFile(psmi, []byte(content), 0644)
-	c.Assert(err, IsNil)
-}
-
 func (d *deviceSuite) TestDeviceFindMountPointErrorBadMountinfo(c *C) {
 	// taken from core18 system
 
@@ -504,8 +545,8 @@ func (d *deviceSuite) TestDeviceFindMountPointErrorBadMountinfo(c *C) {
 	c.Assert(err, IsNil)
 	err = os.Symlink(fakedevice, filepath.Join(d.dir, "/dev/disk/by-label/system-boot"))
 	c.Assert(err, IsNil)
-
-	mockProcSelfFilesystem(c, d.dir, "garbage")
+	restore := osutil.MockMountInfo("garbage")
+	defer restore()
 
 	found, err := gadget.FindMountPointForStructure(&gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
@@ -533,7 +574,8 @@ func (d *deviceSuite) TestDeviceFindMountPointByLabeHappySimple(c *C) {
 170 27 8:2 / /boot/efi rw,relatime shared:58 - vfat ${rootDir}/dev/sda2 rw,fmask=0022,dmask=0022,codepage=437,iocharset=iso8859-1,shortname=mixed,errors=remount-ro
 172 27 8:2 /EFI/ubuntu /boot/grub rw,relatime shared:58 - vfat ${rootDir}/dev/sda2 rw,fmask=0022,dmask=0022,codepage=437,iocharset=iso8859-1,shortname=mixed,errors=remount-ro
 `
-	mockProcSelfFilesystem(c, d.dir, strings.Replace(mountInfo[1:], "${rootDir}", d.dir, -1))
+	restore := osutil.MockMountInfo(strings.Replace(mountInfo[1:], "${rootDir}", d.dir, -1))
+	defer restore()
 
 	found, err := gadget.FindMountPointForStructure(&gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
@@ -562,7 +604,8 @@ func (d *deviceSuite) TestDeviceFindMountPointByLabeHappyReversed(c *C) {
 170 27 8:2 / /boot/efi rw,relatime shared:58 - vfat ${rootDir}/dev/sda2 rw,fmask=0022,dmask=0022,codepage=437,iocharset=iso8859-1,shortname=mixed,errors=remount-ro
 `
 
-	mockProcSelfFilesystem(c, d.dir, strings.Replace(mountInfoReversed[1:], "${rootDir}", d.dir, -1))
+	restore := osutil.MockMountInfo(strings.Replace(mountInfoReversed[1:], "${rootDir}", d.dir, -1))
+	defer restore()
 
 	found, err := gadget.FindMountPointForStructure(&gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
@@ -591,7 +634,8 @@ func (d *deviceSuite) TestDeviceFindMountPointPicksFirstMatch(c *C) {
 170 27 8:2 / /boot/efi rw,relatime shared:58 - vfat ${rootDir}/dev/sda2 rw,fmask=0022,dmask=0022,codepage=437,iocharset=iso8859-1,shortname=mixed,errors=remount-ro
 `
 
-	mockProcSelfFilesystem(c, d.dir, strings.Replace(mountInfo[1:], "${rootDir}", d.dir, -1))
+	restore := osutil.MockMountInfo(strings.Replace(mountInfo[1:], "${rootDir}", d.dir, -1))
+	defer restore()
 
 	found, err := gadget.FindMountPointForStructure(&gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
@@ -615,7 +659,8 @@ func (d *deviceSuite) TestDeviceFindMountPointByPartlabel(c *C) {
 170 27 8:2 / /mount-point rw,relatime shared:58 - ext4 ${rootDir}/dev/fakedevice rw
 `
 
-	mockProcSelfFilesystem(c, d.dir, strings.Replace(mountInfo[1:], "${rootDir}", d.dir, -1))
+	restore := osutil.MockMountInfo(strings.Replace(mountInfo[1:], "${rootDir}", d.dir, -1))
+	defer restore()
 
 	found, err := gadget.FindMountPointForStructure(&gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
@@ -638,7 +683,8 @@ func (d *deviceSuite) TestDeviceFindMountPointChecksFilesystem(c *C) {
 170 27 8:2 / /mount-point rw,relatime shared:58 - vfat ${rootDir}/dev/fakedevice rw
 `
 
-	mockProcSelfFilesystem(c, d.dir, strings.Replace(mountInfo[1:], "${rootDir}", d.dir, -1))
+	restore := osutil.MockMountInfo(strings.Replace(mountInfo[1:], "${rootDir}", d.dir, -1))
+	defer restore()
 
 	found, err := gadget.FindMountPointForStructure(&gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
