@@ -34,7 +34,7 @@ import (
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/gadget"
-	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snap/snapfile"
 	"github.com/snapcore/snapd/snap/snaptest"
 )
 
@@ -124,6 +124,13 @@ defaults:
       bar: baz
 `)
 
+var mockClassicGadgetCoreDefaultsYaml = []byte(`
+defaults:
+  99T7MUlRhtI3U0QFgl5mXXESAiSwt776:
+    ssh:
+      disable: true
+`)
+
 var mockClassicGadgetMultilineDefaultsYaml = []byte(`
 defaults:
   system:
@@ -361,6 +368,54 @@ func (s *gadgetYamlTestSuite) TestReadGadgetYamlOnClassicOnylDefaultsIsValid(c *
 	})
 }
 
+func (s *gadgetYamlTestSuite) TestFlatten(c *C) {
+	cfg := map[string]interface{}{
+		"foo":         "bar",
+		"some.option": true,
+		"sub": map[string]interface{}{
+			"option1": true,
+			"option2": map[string]interface{}{
+				"deep": "2",
+			},
+		},
+	}
+	out := map[string]interface{}{}
+	gadget.Flatten("", cfg, out)
+	c.Check(out, DeepEquals, map[string]interface{}{
+		"foo":              "bar",
+		"some.option":      true,
+		"sub.option1":      true,
+		"sub.option2.deep": "2",
+	})
+}
+
+func (s *gadgetYamlTestSuite) TestCoreConfigDefaults(c *C) {
+	err := ioutil.WriteFile(s.gadgetYamlPath, mockClassicGadgetCoreDefaultsYaml, 0644)
+	c.Assert(err, IsNil)
+
+	ginfo, err := gadget.ReadInfo(s.dir, &modelConstraints{classic: true})
+	c.Assert(err, IsNil)
+	defaults := gadget.SystemDefaults(ginfo.Defaults)
+	c.Check(defaults, DeepEquals, map[string]interface{}{
+		"ssh.disable": true,
+	})
+
+	yaml := string(mockClassicGadgetCoreDefaultsYaml) + `
+  system:
+    something: true
+`
+
+	err = ioutil.WriteFile(s.gadgetYamlPath, []byte(yaml), 0644)
+	c.Assert(err, IsNil)
+	ginfo, err = gadget.ReadInfo(s.dir, &modelConstraints{classic: true})
+	c.Assert(err, IsNil)
+
+	defaults = gadget.SystemDefaults(ginfo.Defaults)
+	c.Check(defaults, DeepEquals, map[string]interface{}{
+		"something": true,
+	})
+}
+
 func (s *gadgetYamlTestSuite) TestReadGadgetDefaultsMultiline(c *C) {
 	err := ioutil.WriteFile(s.gadgetYamlPath, mockClassicGadgetMultilineDefaultsYaml, 0644)
 	c.Assert(err, IsNil)
@@ -1710,7 +1765,7 @@ version: 1.0
 
 func (s *gadgetYamlTestSuite) TestReadGadgetYamlFromSnapFileMissing(c *C) {
 	snapPath := snaptest.MakeTestSnapWithFiles(c, string(mockSnapYaml), nil)
-	snapf, err := snap.Open(snapPath)
+	snapf, err := snapfile.Open(snapPath)
 	c.Assert(err, IsNil)
 
 	// if constraints are nil, we allow a missing gadget.yaml
@@ -1731,7 +1786,7 @@ func (s *gadgetYamlTestSuite) TestReadGadgetYamlFromSnapFileValid(c *C) {
 	snapPath := snaptest.MakeTestSnapWithFiles(c, mockSnapYaml, [][]string{
 		{"meta/gadget.yaml", string(minimalMockGadgetYaml)},
 	})
-	snapf, err := snap.Open(snapPath)
+	snapf, err := snapfile.Open(snapPath)
 	c.Assert(err, IsNil)
 
 	ginfo, err := gadget.ReadInfoFromSnapFile(snapf, nil)
