@@ -64,8 +64,6 @@ func (m *mkfsSuite) TestMkfsExt4Happy(c *C) {
 			"fakeroot",
 			"mkfs.ext4",
 			"-T", "default",
-			"-O", "-metadata_csum",
-			"-O", "uninit_bg",
 			"-d", "contents",
 			"-L", "my-label",
 			"foo.img",
@@ -82,8 +80,6 @@ func (m *mkfsSuite) TestMkfsExt4Happy(c *C) {
 			"fakeroot",
 			"mkfs.ext4",
 			"-T", "default",
-			"-O", "-metadata_csum",
-			"-O", "uninit_bg",
 			"-d", "contents",
 			"foo.img",
 		},
@@ -99,8 +95,6 @@ func (m *mkfsSuite) TestMkfsExt4Happy(c *C) {
 			"fakeroot",
 			"mkfs.ext4",
 			"-T", "default",
-			"-O", "-metadata_csum",
-			"-O", "uninit_bg",
 			"-L", "my-label",
 			"foo.img",
 		},
