@@ -616,14 +616,15 @@ var (
 		"mpris":                   {"app"},
 		"network-manager":         {"app", "core"},
 		"network-manager-observe": {"app", "core"},
-		"network-status":          {"app"},
+		"network-status":          {"core"},
 		"ofono":                   {"app", "core"},
 		"online-accounts-service": {"app"},
-		"ppp":         {"core"},
-		"pulseaudio":  {"app", "core"},
-		"raw-volume":  {"core", "gadget"},
-		"serial-port": {"core", "gadget"},
-		"spi":         {"core", "gadget"},
+		"power-control":           {"core"},
+		"ppp":                     {"core"},
+		"pulseaudio":              {"app", "core"},
+		"raw-volume":              {"core", "gadget"},
+		"serial-port":             {"core", "gadget"},
+		"spi":                     {"core", "gadget"},
 		"storage-framework-service": {"app"},
 		"thumbnailer-service":       {"app"},
 		"ubuntu-download-manager":   {"app"},
@@ -749,15 +750,14 @@ func (s *baseDeclSuite) TestConnection(c *C) {
 	// connecting with these interfaces needs to be allowed on
 	// case-by-case basis
 	noconnect := map[string]bool{
-		"content":                   true,
-		"docker":                    true,
-		"fwupd":                     true,
-		"location-control":          true,
-		"location-observe":          true,
-		"lxd":                       true,
-		"maliit":                    true,
-		"mir":                       true,
-		"network-status":            true,
+		"content":          true,
+		"docker":           true,
+		"fwupd":            true,
+		"location-control": true,
+		"location-observe": true,
+		"lxd":              true,
+		"maliit":           true,
+		"mir":              true,
 		"online-accounts-service":   true,
 		"raw-volume":                true,
 		"storage-framework-service": true,
