@@ -26,11 +26,13 @@ import (
 
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/boot"
+	"github.com/snapcore/snapd/cmd/snap-bootstrap/bootstrap"
 	"github.com/snapcore/snapd/gadget"
 	"github.com/snapcore/snapd/httputil"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/overlord/storecontext"
+	"github.com/snapcore/snapd/sysconfig"
 	"github.com/snapcore/snapd/timings"
 )
 
@@ -94,11 +96,8 @@ func SetLastBecomeOperationalAttempt(m *DeviceManager, t time.Time) {
 	m.lastBecomeOperationalAttempt = t
 }
 
-func SetOperatingMode(m *DeviceManager, mode string) {
-	m.modeEnv.Mode = mode
-}
-func SetRecoverySystem(m *DeviceManager, d string) {
-	m.modeEnv.RecoverySystem = d
+func SetSystemMode(m *DeviceManager, mode string) {
+	m.systemMode = mode
 }
 
 func MockRepeatRequestSerial(label string) (restore func()) {
@@ -152,6 +151,7 @@ func SetBootOkRan(m *DeviceManager, b bool) {
 type (
 	RegistrationContext = registrationContext
 	RemodelContext      = remodelContext
+	SeededSystem        = seededSystem
 )
 
 func RegistrationCtx(m *DeviceManager, t *state.Task) (registrationContext, error) {
@@ -212,6 +212,14 @@ func MockBootMakeBootable(f func(model *asserts.Model, rootdir string, bootWith
 	}
 }
 
+func MockSecbootCheckKeySealingSupported(f func() error) (restore func()) {
+	old := secbootCheckKeySealingSupported
+	secbootCheckKeySealingSupported = f
+	return func() {
+		secbootCheckKeySealingSupported = old
+	}
+}
+
 func MockHttputilNewHTTPClient(f func(opts *httputil.ClientOptions) *http.Client) (restore func()) {
 	old := httputilNewHTTPClient
 	httputilNewHTTPClient = f
@@ -219,3 +227,19 @@ func MockHttputilNewHTTPClient(f func(opts *httputil.ClientOptions) *http.Client
 		httputilNewHTTPClient = old
 	}
 }
+
+func MockSysconfigConfigureRunSystem(f func(opts *sysconfig.Options) error) (restore func()) {
+	old := sysconfigConfigureRunSystem
+	sysconfigConfigureRunSystem = f
+	return func() {
+		sysconfigConfigureRunSystem = old
+	}
+}
+
+func MockBootstrapRun(f func(gadgetRoot, device string, options bootstrap.Options) error) (restore func()) {
+	old := bootstrapRun
+	bootstrapRun = f
+	return func() {
+		bootstrapRun = old
+	}
+}
