@@ -95,7 +95,7 @@ func (b Backend) LinkSnap(info *snap.Info, dev boot.Device, linkCtx LinkContext,
 
 	var err error
 	timings.Run(tm, "generate-wrappers", fmt.Sprintf("generate wrappers for snap %s", info.InstanceName()), func(timings.Measurer) {
-		err = generateWrappers(info, linkCtx.PrevDisabledServices)
+		err = b.generateWrappers(info, linkCtx.PrevDisabledServices)
 	})
 	if err != nil {
 		return false, err
@@ -154,7 +154,7 @@ func (b Backend) StopServices(apps []*snap.AppInfo, reason snap.ServiceStopReaso
 	return wrappers.StopServices(apps, reason, meter, tm)
 }
 
-func generateWrappers(s *snap.Info, disabledSvcs []string) error {
+func (b Backend) generateWrappers(s *snap.Info, disabledSvcs []string) error {
 	var err error
 	var cleanupFuncs []func(*snap.Info) error
 	defer func() {
@@ -177,7 +177,8 @@ func generateWrappers(s *snap.Info, disabledSvcs []string) error {
 	cleanupFuncs = append(cleanupFuncs, wrappers.RemoveSnapBinaries)
 
 	// add the daemons from the snap.yaml
-	if err = wrappers.AddSnapServices(s, disabledSvcs, progress.Null); err != nil {
+	opts := &wrappers.AddSnapServicesOptions{Preseeding: b.preseed}
+	if err = wrappers.AddSnapServices(s, disabledSvcs, opts, progress.Null); err != nil {
 		return err
 	}
 	cleanupFuncs = append(cleanupFuncs, func(s *snap.Info) error {
