@@ -19,6 +19,14 @@
 
 package seed
 
+import (
+	"github.com/snapcore/snapd/seed/internal"
+)
+
+type InternalSnap16 = internal.Snap16
+
 var (
 	LoadAssertions = loadAssertions
+
+	ValidateUC20SeedSystemLabel = validateUC20SeedSystemLabel
 )
