@@ -28,6 +28,7 @@ import (
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/asserts/sysdb"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snap/snapfile"
 )
 
 var trusted = sysdb.Trusted()
@@ -94,7 +95,7 @@ func readAsserts(batch *asserts.Batch, fn string) ([]*asserts.Ref, error) {
 }
 
 func readInfo(snapPath string, si *snap.SideInfo) (*snap.Info, error) {
-	snapf, err := snap.Open(snapPath)
+	snapf, err := snapfile.Open(snapPath)
 	if err != nil {
 		return nil, err
 	}
@@ -117,3 +118,39 @@ func snapTypeFromModel(modSnap *asserts.ModelSnap) snap.Type {
 		return snap.TypeApp
 	}
 }
+
+func essentialSnapTypesToModelFilter(essentialTypes []snap.Type) func(modSnap *asserts.ModelSnap) bool {
+	m := make(map[string]bool, len(essentialTypes))
+	for _, t := range essentialTypes {
+		switch t {
+		case snap.TypeBase:
+			m["base"] = true
+		case snap.TypeOS:
+			m["core"] = true
+		case snap.TypeGadget:
+			m["gadget"] = true
+		case snap.TypeKernel:
+			m["kernel"] = true
+		case snap.TypeSnapd:
+			m["snapd"] = true
+		}
+	}
+
+	return func(modSnap *asserts.ModelSnap) bool {
+		return m[modSnap.SnapType]
+	}
+}
+
+func findBrand(seed Seed, db asserts.RODatabase) (*asserts.Account, error) {
+	model, err := seed.Model()
+	if err != nil {
+		return nil, err
+	}
+	a, err := db.Find(asserts.AccountType, map[string]string{
+		"account-id": model.BrandID(),
+	})
+	if err != nil {
+		return nil, fmt.Errorf("internal error: %v", err)
+	}
+	return a.(*asserts.Account), nil
+}
