@@ -112,6 +112,10 @@ func (s *seed16) Model() (*asserts.Model, error) {
 	return s.model, nil
 }
 
+func (s *seed16) Brand() (*asserts.Account, error) {
+	return findBrand(s, s.db)
+}
+
 func (s *seed16) addSnap(sn *internal.Snap16, pinnedTrack string, tm timings.Measurer) (*Snap, error) {
 	path := filepath.Join(s.seedDir, "snaps", sn.File)
 	snapChannel := sn.Channel
@@ -157,6 +161,14 @@ func (s *seed16) addSnap(sn *internal.Snap16, pinnedTrack string, tm timings.Mea
 	return seedSnap, nil
 }
 
+type essentialSnapMissingError struct {
+	SnapName string
+}
+
+func (e *essentialSnapMissingError) Error() string {
+	return fmt.Sprintf("essential snap %q required by the model is missing in the seed", e.SnapName)
+}
+
 func (s *seed16) LoadMeta(tm timings.Measurer) error {
 	model, err := s.Model()
 	if err != nil {
@@ -181,7 +193,9 @@ func (s *seed16) LoadMeta(tm timings.Measurer) error {
 	}
 	added := make(map[string]bool, 3)
 	classic := model.Classic()
-	_, s.usesSnapdSnap = seeding["snapd"]
+	_, usesSnapdSnap := seeding["snapd"]
+	usesSnapdSnap = usesSnapdSnap || required.Contains(naming.Snap("snapd"))
+	s.usesSnapdSnap = usesSnapdSnap
 
 	baseSnap := "core"
 	classicWithSnapd := false
@@ -203,7 +217,7 @@ func (s *seed16) LoadMeta(tm timings.Measurer) error {
 		}
 		yamlSnap := seeding[snapName]
 		if yamlSnap == nil {
-			return nil, fmt.Errorf("essential snap %q required by the model is missing in the seed", snapName)
+			return nil, &essentialSnapMissingError{SnapName: snapName}
 		}
 
 		seedSnap, err := s.addSnap(yamlSnap, pinnedTrack, tm)
