@@ -25,8 +25,20 @@ import (
 	"io/ioutil"
 	"os"
 	"path/filepath"
+
+	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/snap"
 )
 
+func IsSnapDir(path string) bool {
+	if osutil.IsDirectory(path) {
+		if osutil.FileExists(filepath.Join(path, "meta", "snap.yaml")) {
+			return true
+		}
+	}
+	return false
+}
+
 // SnapDir is the snapdir based snap.
 type SnapDir struct {
 	path string
@@ -53,10 +65,19 @@ func (s *SnapDir) Size() (size int64, err error) {
 	return totalSize, nil
 }
 
-func (s *SnapDir) Install(targetPath, mountDir string) (bool, error) {
+func (s *SnapDir) Install(targetPath, mountDir string, opts *snap.InstallOptions) (bool, error) {
+	// TODO:UC20: support MustNotCrossDevices somehow here
+
 	return false, os.Symlink(s.path, targetPath)
 }
 
+func (s *SnapDir) RandomAccessFile(file string) (interface {
+	io.ReaderAt
+	io.Closer
+}, error) {
+	return os.Open(filepath.Join(s.path, file))
+}
+
 func (s *SnapDir) ReadFile(file string) (content []byte, err error) {
 	return ioutil.ReadFile(filepath.Join(s.path, file))
 }
