@@ -24,7 +24,6 @@ import (
 	"io/ioutil"
 	"os"
 	"os/user"
-	"strings"
 	"testing"
 
 	. "gopkg.in/check.v1"
@@ -32,6 +31,7 @@ import (
 	"github.com/snapcore/snapd/arch"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/features"
+	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/sys"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/testutil"
@@ -81,30 +81,27 @@ func (s *HTestSuite) TearDownTest(c *C) {
 
 func (ts *HTestSuite) TestBasic(c *C) {
 	env := basicEnv(mockSnapInfo)
-
-	c.Assert(env, DeepEquals, map[string]string{
+	c.Assert(env, DeepEquals, osutil.Environment{
 		"SNAP":               fmt.Sprintf("%s/foo/17", dirs.CoreSnapMountDir),
-		"SNAP_ARCH":          arch.DpkgArchitecture(),
 		"SNAP_COMMON":        "/var/snap/foo/common",
 		"SNAP_DATA":          "/var/snap/foo/17",
-		"SNAP_LIBRARY_PATH":  "/var/lib/snapd/lib/gl:/var/lib/snapd/lib/gl32:/var/lib/snapd/void",
 		"SNAP_NAME":          "foo",
 		"SNAP_INSTANCE_NAME": "foo",
 		"SNAP_INSTANCE_KEY":  "",
-		"SNAP_REEXEC":        "",
-		"SNAP_REVISION":      "17",
 		"SNAP_VERSION":       "1.0",
+		"SNAP_REVISION":      "17",
+		"SNAP_ARCH":          arch.DpkgArchitecture(),
+		"SNAP_LIBRARY_PATH":  "/var/lib/snapd/lib/gl:/var/lib/snapd/lib/gl32:/var/lib/snapd/void",
+		"SNAP_REEXEC":        "",
 	})
-
 }
 
 func (ts *HTestSuite) TestUser(c *C) {
 	env := userEnv(mockSnapInfo, "/root")
-
-	c.Assert(env, DeepEquals, map[string]string{
-		"HOME":             "/root/snap/foo/17",
+	c.Assert(env, DeepEquals, osutil.Environment{
 		"SNAP_USER_COMMON": "/root/snap/foo/common",
 		"SNAP_USER_DATA":   "/root/snap/foo/17",
+		"HOME":             "/root/snap/foo/17",
 		"XDG_RUNTIME_DIR":  fmt.Sprintf("/run/user/%d/snap.foo", sys.Geteuid()),
 	})
 }
@@ -117,7 +114,7 @@ func (ts *HTestSuite) TestUserForClassicConfinement(c *C) {
 	// With the classic-preserves-xdg-runtime-dir feature disabled the snap
 	// per-user environment contains an override for XDG_RUNTIME_DIR.
 	env := userEnv(mockClassicSnapInfo, "/root")
-	c.Assert(env, DeepEquals, map[string]string{
+	c.Assert(env, DeepEquals, osutil.Environment{
 		// NOTE: Both HOME and XDG_RUNTIME_DIR are not defined here.
 		"SNAP_USER_COMMON": "/root/snap/foo/common",
 		"SNAP_USER_DATA":   "/root/snap/foo/17",
@@ -129,12 +126,11 @@ func (ts *HTestSuite) TestUserForClassicConfinement(c *C) {
 	f := features.ClassicPreservesXdgRuntimeDir
 	c.Assert(ioutil.WriteFile(f.ControlFile(), nil, 0644), IsNil)
 	env = userEnv(mockClassicSnapInfo, "/root")
-	c.Assert(env, DeepEquals, map[string]string{
+	c.Assert(env, DeepEquals, osutil.Environment{
 		// NOTE: Both HOME and XDG_RUNTIME_DIR are not defined here.
 		"SNAP_USER_COMMON": "/root/snap/foo/common",
 		"SNAP_USER_DATA":   "/root/snap/foo/17",
 	})
-
 }
 
 func (s *HTestSuite) TestSnapRunSnapExecEnv(c *C) {
@@ -154,24 +150,22 @@ func (s *HTestSuite) TestSnapRunSnapExecEnv(c *C) {
 		}
 
 		env := snapEnv(info)
-		c.Check(env, DeepEquals, map[string]string{
-			"SNAP_ARCH":          arch.DpkgArchitecture(),
-			"SNAP_LIBRARY_PATH":  "/var/lib/snapd/lib/gl:/var/lib/snapd/lib/gl32:/var/lib/snapd/void",
+		c.Assert(env, DeepEquals, osutil.Environment{
+			"SNAP":               fmt.Sprintf("%s/snapname/42", dirs.CoreSnapMountDir),
+			"SNAP_COMMON":        "/var/snap/snapname/common",
+			"SNAP_DATA":          "/var/snap/snapname/42",
 			"SNAP_NAME":          "snapname",
 			"SNAP_INSTANCE_NAME": "snapname",
 			"SNAP_INSTANCE_KEY":  "",
-			"SNAP_REEXEC":        "",
-			"SNAP_REVISION":      "42",
 			"SNAP_VERSION":       "1.0",
-
-			"SNAP":        fmt.Sprintf("%s/snapname/42", dirs.CoreSnapMountDir),
-			"SNAP_COMMON": "/var/snap/snapname/common",
-			"SNAP_DATA":   "/var/snap/snapname/42",
-
-			"SNAP_USER_COMMON": fmt.Sprintf("%s/snap/snapname/common", usr.HomeDir),
-			"SNAP_USER_DATA":   fmt.Sprintf("%s/snap/snapname/42", usr.HomeDir),
-			"XDG_RUNTIME_DIR":  fmt.Sprintf("/run/user/%d/snap.snapname", sys.Geteuid()),
-			"HOME":             fmt.Sprintf("%s/snap/snapname/42", usr.HomeDir),
+			"SNAP_REVISION":      "42",
+			"SNAP_ARCH":          arch.DpkgArchitecture(),
+			"SNAP_LIBRARY_PATH":  "/var/lib/snapd/lib/gl:/var/lib/snapd/lib/gl32:/var/lib/snapd/void",
+			"SNAP_REEXEC":        "",
+			"SNAP_USER_COMMON":   fmt.Sprintf("%s/snap/snapname/common", usr.HomeDir),
+			"SNAP_USER_DATA":     fmt.Sprintf("%s/snap/snapname/42", usr.HomeDir),
+			"HOME":               fmt.Sprintf("%s/snap/snapname/42", usr.HomeDir),
+			"XDG_RUNTIME_DIR":    fmt.Sprintf("/run/user/%d/snap.snapname", sys.Geteuid()),
 		})
 	}
 }
@@ -196,28 +190,26 @@ func (s *HTestSuite) TestParallelInstallSnapRunSnapExecEnv(c *C) {
 		}
 
 		env := snapEnv(info)
-		c.Check(env, DeepEquals, map[string]string{
-			"SNAP_ARCH":          arch.DpkgArchitecture(),
-			"SNAP_LIBRARY_PATH":  "/var/lib/snapd/lib/gl:/var/lib/snapd/lib/gl32:/var/lib/snapd/void",
+		c.Check(env, DeepEquals, osutil.Environment{
+			// Those are mapped to snap-specific directories by
+			// mount namespace setup
+			"SNAP":               fmt.Sprintf("%s/snapname/42", dirs.CoreSnapMountDir),
+			"SNAP_COMMON":        "/var/snap/snapname/common",
+			"SNAP_DATA":          "/var/snap/snapname/42",
 			"SNAP_NAME":          "snapname",
 			"SNAP_INSTANCE_NAME": "snapname_foo",
 			"SNAP_INSTANCE_KEY":  "foo",
-			"SNAP_REEXEC":        "",
-			"SNAP_REVISION":      "42",
 			"SNAP_VERSION":       "1.0",
-
-			// Those are mapped to snap-specific directories by
-			// mount namespace setup
-			"SNAP":        fmt.Sprintf("%s/snapname/42", dirs.CoreSnapMountDir),
-			"SNAP_COMMON": "/var/snap/snapname/common",
-			"SNAP_DATA":   "/var/snap/snapname/42",
-
+			"SNAP_REVISION":      "42",
+			"SNAP_ARCH":          arch.DpkgArchitecture(),
+			"SNAP_LIBRARY_PATH":  "/var/lib/snapd/lib/gl:/var/lib/snapd/lib/gl32:/var/lib/snapd/void",
+			"SNAP_REEXEC":        "",
 			// User's data directories are not mapped to
 			// snap-specific ones
 			"SNAP_USER_COMMON": fmt.Sprintf("%s/snap/snapname_foo/common", usr.HomeDir),
 			"SNAP_USER_DATA":   fmt.Sprintf("%s/snap/snapname_foo/42", usr.HomeDir),
-			"XDG_RUNTIME_DIR":  fmt.Sprintf("/run/user/%d/snap.snapname_foo", sys.Geteuid()),
 			"HOME":             fmt.Sprintf("%s/snap/snapname_foo/42", usr.HomeDir),
+			"XDG_RUNTIME_DIR":  fmt.Sprintf("/run/user/%d/snap.snapname_foo", sys.Geteuid()),
 		})
 	}
 }
@@ -227,10 +219,10 @@ func (ts *HTestSuite) TestParallelInstallUser(c *C) {
 	info.InstanceKey = "bar"
 	env := userEnv(&info, "/root")
 
-	c.Assert(env, DeepEquals, map[string]string{
-		"HOME":             "/root/snap/foo_bar/17",
+	c.Assert(env, DeepEquals, osutil.Environment{
 		"SNAP_USER_COMMON": "/root/snap/foo_bar/common",
 		"SNAP_USER_DATA":   "/root/snap/foo_bar/17",
+		"HOME":             "/root/snap/foo_bar/17",
 		"XDG_RUNTIME_DIR":  fmt.Sprintf("/run/user/%d/snap.foo_bar", sys.Geteuid()),
 	})
 }
@@ -246,7 +238,7 @@ func (ts *HTestSuite) TestParallelInstallUserForClassicConfinement(c *C) {
 	// With the classic-preserves-xdg-runtime-dir feature disabled the snap
 	// per-user environment contains an override for XDG_RUNTIME_DIR.
 	env := userEnv(&info, "/root")
-	c.Assert(env, DeepEquals, map[string]string{
+	c.Assert(env, DeepEquals, osutil.Environment{
 		"SNAP_USER_COMMON": "/root/snap/foo_bar/common",
 		"SNAP_USER_DATA":   "/root/snap/foo_bar/17",
 		"XDG_RUNTIME_DIR":  fmt.Sprintf(dirs.GlobalRootDir+"/run/user/%d/snap.foo_bar", sys.Geteuid()),
@@ -257,65 +249,33 @@ func (ts *HTestSuite) TestParallelInstallUserForClassicConfinement(c *C) {
 	f := features.ClassicPreservesXdgRuntimeDir
 	c.Assert(ioutil.WriteFile(f.ControlFile(), nil, 0644), IsNil)
 	env = userEnv(&info, "/root")
-	c.Assert(env, DeepEquals, map[string]string{
-		// NOTE: Both HOME and XDG_RUNTIME_DIR are not defined here.
+	c.Assert(env, DeepEquals, osutil.Environment{
+		// NOTE, Both HOME and XDG_RUNTIME_DIR are not defined here.
 		"SNAP_USER_COMMON": "/root/snap/foo_bar/common",
 		"SNAP_USER_DATA":   "/root/snap/foo_bar/17",
 	})
 }
 
-func envValue(env []string, key string) (bool, string) {
-	for _, item := range env {
-		if strings.HasPrefix(item, key+"=") {
-			return true, strings.SplitN(item, "=", 2)[1]
-		}
-	}
-	return false, ""
+func (s *HTestSuite) TestExtendEnvForRunForNonClassic(c *C) {
+	env := osutil.Environment{"TMPDIR": "/var/tmp"}
+
+	ExtendEnvForRun(env, mockSnapInfo)
+
+	c.Assert(env["SNAP_NAME"], Equals, "foo")
+	c.Assert(env["SNAP_COMMON"], Equals, "/var/snap/foo/common")
+	c.Assert(env["SNAP_DATA"], Equals, "/var/snap/foo/17")
+
+	c.Assert(env["TMPDIR"], Equals, "/var/tmp")
 }
 
-func (s *HTestSuite) TestExtraEnvForExecEnv(c *C) {
-	info, err := snap.InfoFromSnapYaml(mockYaml)
-	c.Assert(err, IsNil)
-	info.SideInfo.Revision = snap.R(42)
+func (s *HTestSuite) TestExtendEnvForRunForClassic(c *C) {
+	env := osutil.Environment{"TMPDIR": "/var/tmp"}
 
-	env := ExecEnv(info, map[string]string{"FOO": "BAR"})
-	found, val := envValue(env, "FOO")
-	c.Assert(found, Equals, true)
-	c.Assert(val, Equals, "BAR")
-}
+	ExtendEnvForRun(env, mockClassicSnapInfo)
 
-func setenvWithReset(s *HTestSuite, key string, val string) {
-	tmpdirEnv, tmpdirFound := os.LookupEnv("TMPDIR")
-	os.Setenv("TMPDIR", "/var/tmp")
-	if tmpdirFound {
-		s.AddCleanup(func() { os.Setenv("TMPDIR", tmpdirEnv) })
-	} else {
-		s.AddCleanup(func() { os.Unsetenv("TMPDIR") })
-	}
-}
+	c.Assert(env["SNAP_NAME"], Equals, "foo")
+	c.Assert(env["SNAP_COMMON"], Equals, "/var/snap/foo/common")
+	c.Assert(env["SNAP_DATA"], Equals, "/var/snap/foo/17")
 
-func (s *HTestSuite) TestExecEnvNoRenameTMPDIRForNonClassic(c *C) {
-	setenvWithReset(s, "TMPDIR", "/var/tmp")
-
-	env := ExecEnv(mockSnapInfo, map[string]string{})
-
-	found, val := envValue(env, "TMPDIR")
-	c.Assert(found, Equals, true)
-	c.Assert(val, Equals, "/var/tmp")
-
-	found, _ = envValue(env, PreservedUnsafePrefix+"TMPDIR")
-	c.Assert(found, Equals, false)
-}
-
-func (s *HTestSuite) TestExecEnvRenameTMPDIRForClassic(c *C) {
-	setenvWithReset(s, "TMPDIR", "/var/tmp")
-
-	env := ExecEnv(mockClassicSnapInfo, map[string]string{})
-
-	found, _ := envValue(env, "TMPDIR")
-	c.Assert(found, Equals, false)
-
-	found, val := envValue(env, PreservedUnsafePrefix+"TMPDIR")
-	c.Assert(found, Equals, true)
-	c.Assert(val, Equals, "/var/tmp")
+	c.Assert(env["TMPDIR"], Equals, "/var/tmp")
 }
