@@ -195,33 +195,40 @@ func AssertedSnapID(snapName string) string {
 // snap.yaml content and optional extras files specified as pairs of
 // relative file path and its content.
 func MakeTestSnapWithFiles(c *check.C, snapYamlContent string, files [][]string) (snapFilePath string) {
+	path, _ := MakeTestSnapInfoWithFiles(c, snapYamlContent, files, nil)
+	return path
+}
+
+// MakeTestSnapInfoWithFiles makes a squashfs snap file with the given snap.yaml
+// content and optional extra files specified as pairs of relative file path and
+// it's contents, and returns the path to the snap file and a suitable snap.Info
+// for the snap
+func MakeTestSnapInfoWithFiles(c *check.C, snapYamlContent string, files [][]string, si *snap.SideInfo) (snapFilePath string, info *snap.Info) {
 	tmpdir := c.MkDir()
 	snapSource := filepath.Join(tmpdir, "snapsrc")
-
 	err := os.MkdirAll(filepath.Join(snapSource, "meta"), 0755)
-	if err != nil {
-		panic(err)
-	}
+	c.Assert(err, check.IsNil)
 	snapYamlFn := filepath.Join(snapSource, "meta", "snap.yaml")
 	err = ioutil.WriteFile(snapYamlFn, []byte(snapYamlContent), 0644)
-	if err != nil {
-		panic(err)
-	}
-
+	c.Assert(err, check.IsNil)
 	PopulateDir(snapSource, files)
-
 	restoreSanitize := snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {})
 	defer restoreSanitize()
 
+	// Parse the yaml (we need the Name).
+	snapInfo, err := snap.InfoFromSnapYaml([]byte(snapYamlContent))
+	c.Assert(err, check.IsNil)
+	if si != nil {
+		snapInfo.SideInfo = *si
+	}
 	err = osutil.ChDir(snapSource, func() error {
 		var err error
 		snapFilePath, err = pack.Snap(snapSource, nil)
 		return err
 	})
-	if err != nil {
-		panic(err)
-	}
-	return filepath.Join(snapSource, snapFilePath)
+	c.Assert(err, check.IsNil)
+	return filepath.Join(snapSource, snapFilePath), snapInfo
+
 }
 
 // MustParseChannel parses a string representing a store channel and
