@@ -391,6 +391,42 @@ func (s *SystemdTestSuite) TestAvailable(c *C) {
 	c.Check(s.argses, DeepEquals, [][]string{{"--version"}})
 }
 
+func (s *SystemdTestSuite) TestVersion(c *C) {
+	s.outs = [][]byte{
+		[]byte("systemd 223\n+PAM\n"),
+		[]byte("systemd 245 (245.4-4ubuntu3)\n+PAM +AUDIT +SELINUX +IMA\n"),
+		// error cases
+		[]byte("foo 223\n+PAM\n"),
+		[]byte(""),
+		[]byte("systemd abc\n+PAM\n"),
+	}
+
+	v, err := Version()
+	c.Assert(err, IsNil)
+	c.Check(v, Equals, 223)
+
+	v, err = Version()
+	c.Assert(err, IsNil)
+	c.Check(v, Equals, 245)
+
+	_, err = Version()
+	c.Assert(err, ErrorMatches, `cannot parse systemd version: expected "systemd", got "foo"`)
+
+	_, err = Version()
+	c.Assert(err, ErrorMatches, `cannot read systemd version: <nil>`)
+
+	_, err = Version()
+	c.Assert(err, ErrorMatches, `cannot convert systemd version to number: abc`)
+
+	c.Check(s.argses, DeepEquals, [][]string{
+		{"--version"},
+		{"--version"},
+		{"--version"},
+		{"--version"},
+		{"--version"},
+	})
+}
+
 func (s *SystemdTestSuite) TestEnable(c *C) {
 	err := New("xyzzy", SystemMode, s.rep).Enable("foo")
 	c.Assert(err, IsNil)
@@ -575,6 +611,8 @@ WantedBy=multi-user.target
 func (s *SystemdTestSuite) TestWriteSELinuxMountUnit(c *C) {
 	restore := selinux.MockIsEnabled(func() (bool, error) { return true, nil })
 	defer restore()
+	restore = selinux.MockIsEnforcing(func() (bool, error) { return true, nil })
+	defer restore()
 	restore = squashfs.MockNeedsFuse(false)
 	defer restore()
 
@@ -765,6 +803,9 @@ func makeMockMountUnit(c *C, mountDir string) string {
 func (s *SystemdTestSuite) TestRemoveMountUnit(c *C) {
 	rootDir := dirs.GlobalRootDir
 
+	restore := osutil.MockMountInfo("")
+	defer restore()
+
 	mountDir := rootDir + "/snap/foo/42"
 	mountUnit := makeMockMountUnit(c, mountDir)
 	err := New(rootDir, SystemMode, nil).RemoveMountUnitFile(mountDir)
@@ -864,7 +905,7 @@ WantedBy=multi-user.target
 `
 
 func (s *SystemdTestSuite) TestPreseedModeAddMountUnit(c *C) {
-	sysd := NewEmulationMode()
+	sysd := NewEmulationMode(dirs.GlobalRootDir)
 
 	restore := squashfs.MockNeedsFuse(false)
 	defer restore()
@@ -889,7 +930,7 @@ func (s *SystemdTestSuite) TestPreseedModeAddMountUnit(c *C) {
 }
 
 func (s *SystemdTestSuite) TestPreseedModeAddMountUnitWithFuse(c *C) {
-	sysd := NewEmulationMode()
+	sysd := NewEmulationMode(dirs.GlobalRootDir)
 
 	restore := MockSquashFsType(func() (string, []string, error) { return "fuse.squashfuse", []string{"a,b,c"}, nil })
 	defer restore()
@@ -909,7 +950,7 @@ func (s *SystemdTestSuite) TestPreseedModeAddMountUnitWithFuse(c *C) {
 }
 
 func (s *SystemdTestSuite) TestPreseedModeMountError(c *C) {
-	sysd := NewEmulationMode()
+	sysd := NewEmulationMode(dirs.GlobalRootDir)
 
 	restore := squashfs.MockNeedsFuse(false)
 	defer restore()
@@ -936,7 +977,7 @@ func (s *SystemdTestSuite) TestPreseedModeRemoveMountUnit(c *C) {
 	mockUmountCmd := testutil.MockCommand(c, "umount", "")
 	defer mockUmountCmd.Restore()
 
-	sysd := NewEmulationMode()
+	sysd := NewEmulationMode(dirs.GlobalRootDir)
 
 	mountUnit := makeMockMountUnit(c, mountDir)
 	symlinkPath := filepath.Join(dirs.SnapServicesDir, "multi-user.target.wants", filepath.Base(mountUnit))
@@ -965,7 +1006,7 @@ func (s *SystemdTestSuite) TestPreseedModeRemoveMountUnitUnmounted(c *C) {
 	mockUmountCmd := testutil.MockCommand(c, "umount", "")
 	defer mockUmountCmd.Restore()
 
-	sysd := NewEmulationMode()
+	sysd := NewEmulationMode(dirs.GlobalRootDir)
 	mountUnit := makeMockMountUnit(c, mountDir)
 	symlinkPath := filepath.Join(dirs.SnapServicesDir, "multi-user.target.wants", filepath.Base(mountUnit))
 	c.Assert(os.Symlink(mountUnit, symlinkPath), IsNil)
@@ -983,7 +1024,7 @@ func (s *SystemdTestSuite) TestPreseedModeRemoveMountUnitUnmounted(c *C) {
 }
 
 func (s *SystemdTestSuite) TestPreseedModeBindmountNotSupported(c *C) {
-	sysd := NewEmulationMode()
+	sysd := NewEmulationMode(dirs.GlobalRootDir)
 
 	restore := squashfs.MockNeedsFuse(false)
 	defer restore()
@@ -993,3 +1034,38 @@ func (s *SystemdTestSuite) TestPreseedModeBindmountNotSupported(c *C) {
 	_, err := sysd.AddMountUnitFile("foo", "42", mockSnapPath, "/snap/snapname/123", "")
 	c.Assert(err, ErrorMatches, `bind-mounted directory is not supported in emulation mode`)
 }
+
+func (s *SystemdTestSuite) TestEnableInEmulationMode(c *C) {
+	sysd := NewEmulationMode("/path")
+	c.Assert(sysd.Enable("foo"), IsNil)
+
+	sysd = NewEmulationMode("")
+	c.Assert(sysd.Enable("bar"), IsNil)
+	c.Check(s.argses, DeepEquals, [][]string{
+		{"--root", "/path", "enable", "foo"},
+		{"--root", dirs.GlobalRootDir, "enable", "bar"}})
+}
+
+func (s *SystemdTestSuite) TestDisableInEmulationMode(c *C) {
+	sysd := NewEmulationMode("/path")
+	c.Assert(sysd.Disable("foo"), IsNil)
+
+	c.Check(s.argses, DeepEquals, [][]string{
+		{"--root", "/path", "disable", "foo"}})
+}
+
+func (s *SystemdTestSuite) TestMaskInEmulationMode(c *C) {
+	sysd := NewEmulationMode("/path")
+	c.Assert(sysd.Mask("foo"), IsNil)
+
+	c.Check(s.argses, DeepEquals, [][]string{
+		{"--root", "/path", "mask", "foo"}})
+}
+
+func (s *SystemdTestSuite) TestUnmaskInEmulationMode(c *C) {
+	sysd := NewEmulationMode("/path")
+	c.Assert(sysd.Unmask("foo"), IsNil)
+
+	c.Check(s.argses, DeepEquals, [][]string{
+		{"--root", "/path", "unmask", "foo"}})
+}
